/***
*
*	Copyright (c) 1996-2002, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/
// Rukia: I modded the name, and replaced REAL with "REAL", so that PAWN on linux64 extends to double correctly.
// 		However, it is still essentially identical to the original HL1 Vector class, but renamed.
#ifndef VECTOR_H
#define VECTOR_H

#include "amxxmodule.h"
#include <math.h>

//=========================================================
// 2DVector - used for many pathfinding and many other 
// operations that are treated as planar rather than 3d.
//=========================================================
class HLVector2D
{
public:
	inline HLVector2D(void)									{ }
	inline HLVector2D(REAL X, REAL Y)						{ x = X; y = Y; }
	inline HLVector2D operator+(const HLVector2D& v)	const	{ return HLVector2D(x+v.x, y+v.y);	}
	inline HLVector2D operator-(const HLVector2D& v)	const	{ return HLVector2D(x-v.x, y-v.y);	}
	inline HLVector2D operator*(REAL fl)				const	{ return HLVector2D(x*fl, y*fl);	}
	inline HLVector2D operator/(REAL fl)				const	{ return HLVector2D(x/fl, y/fl);	}
	
	inline REAL Length(void)						const	{ return sqrt(x*x + y*y );		}

	inline HLVector2D Normalize ( void ) const
	{
		HLVector2D vec2;

		REAL flLen = Length();
		if ( flLen == 0 )
		{
			return HLVector2D( 0, 0 );
		}
		else
		{
			flLen = 1 / flLen;
			return HLVector2D( x * flLen, y * flLen );
		}
	}

	REAL x, y;
};

inline REAL DotProduct(const HLVector2D& a, const HLVector2D& b) { return( a.x*b.x + a.y*b.y ); }
inline HLVector2D operator*(REAL fl, const HLVector2D& v)	{ return v * fl; }

//=========================================================
// 3D Vector
//=========================================================
class HLVector						// same data-layout as engine's vec3_t,
{								//		which is a vec_t[3]
public:
	// Construction/destruction
	inline HLVector(void)								{ }
	inline HLVector(REAL X, REAL Y, REAL Z)		{ x = X; y = Y; z = Z;						}
	//inline HLVector(REAL X, REAL Y, REAL Z)		{ x = (REAL)X; y = (REAL)Y; z = (REAL)Z;	}
	//inline HLVector(int X, int Y, int Z)				{ x = (REAL)X; y = (REAL)Y; z = (REAL)Z;	}
	inline HLVector(const HLVector& v)					{ x = v.x; y = v.y; z = v.z;				}
	inline HLVector(const HLVector* v)					{ x = v->x; y = v->y; z = v->z;				} 
	inline HLVector(REAL rgfl[3])					{ x = rgfl[0]; y = rgfl[1]; z = rgfl[2];	}

	// Operators
	inline HLVector operator-(void) const				{ return HLVector(-x,-y,-z);				}
	inline bool operator==(const HLVector& v) const	{ return x==v.x && y==v.y && z==v.z;	}
	inline bool operator!=(const HLVector& v) const	{ return !(*this==v);					}
	inline bool operator<(const HLVector& v) const { return (x < v.x) && (y < v.y) && (z < v.z); }
	inline HLVector operator+(const HLVector& v) const	{ return HLVector(x+v.x, y+v.y, z+v.z);	}
	inline HLVector operator-(const HLVector& v) const	{ return HLVector(x-v.x, y-v.y, z-v.z);	}
	inline HLVector operator*(REAL fl) const			{ return HLVector(x*fl, y*fl, z*fl);		}
	inline HLVector operator/(REAL fl) const			{ return HLVector(x/fl, y/fl, z/fl);		}
	
	// Methods
	inline void CopyToArray(REAL* rgfl) const		{ rgfl[0] = x, rgfl[1] = y, rgfl[2] = z; }
	inline REAL Length(void) const					{ return sqrt(x*x + y*y + z*z); }
	operator REAL *()								{ return &x; } // HLVectors will now automatically convert to REAL * when needed
	operator const REAL *() const					{ return &x; } // HLVectors will now automatically convert to REAL * when needed
	inline HLVector Normalize(void) const
	{
		REAL flLen = Length();
		if (flLen == 0) return HLVector(0,0,1); // ????
		flLen = 1 / flLen;
		return HLVector(x * flLen, y * flLen, z * flLen);
	}

	inline HLVector2D Make2D ( void ) const
	{
		HLVector2D	Vec2;

		Vec2.x = x;
		Vec2.y = y;

		return Vec2;
	}
	inline REAL Length2D(void) const					{ return sqrt(x*x + y*y); }

	// Members
	REAL x, y, z;
};
inline HLVector operator*(REAL fl, const HLVector& v)	{ return v * fl; }
inline REAL DotProduct(const HLVector& a, const HLVector& b) { return(a.x*b.x+a.y*b.y+a.z*b.z); }
inline HLVector CrossProduct(const HLVector& a, const HLVector& b) { return HLVector( a.y*b.z - a.z*b.y, a.z*b.x - a.x*b.z, a.x*b.y - a.y*b.x ); }

#endif
