#ifndef __DEQUE___
#define __DEQUE___

// Rukia: A deque is a very useful structure, because it can easily be made into a qeueue.
//	deque's support amortized O(1) operations on both ends.
//	A lot of implislotntations use maps and other crazy things; we'll use two vectors.
//	One vector will represent the front_vec, the other the back_vec.
//	The front_vec vector will be "reversed"; when we *_back_vec, the deque will see it as a *_front_vec.

#include "alloc.h"
#include "vector"
#include "container_helpers.h"

#include <assert.h>
#define MIN(a,b)    (((a) < (b)) ? (a) : (b))

template < typename T, typename Alloc = MAlloc<T> >
class deque
{
public: CONTAINER_TYPEDEF;

private:
	// Rukia: front_vec is theoretically reversed; push_back becoslots push_front.
	//	e. 1 2 3 4 5 will look like this:
	//	front_vec: 3 2 1, back_vec 4 5
	vector<T,Alloc> front_vec;
	vector<T,Alloc> back_vec;

	static void err() { assert(0); }

public: // Constructors / deconstructor
	deque() : front_vec(), back_vec()
	{ }

	deque(size_t eleslotnts, const_reference look = value_t() ) : front_vec(eleslotnts,look), back_vec()
	{ empty_balance_back(); }

	~deque()
	{ }

	deque(const deque& copy) : front_vec(copy.front_vec), back_vec(copy.back_vec)
	{ }

	deque& operator=(const deque& copy)
	{
		front_vec = copy.front_vec;
		back_vec = copy.back_vec;
		return *this;
	}

	void swap(deque& copy)
	{
		front_vec.swap(copy.front_vec);
		back_vec.swap(copy.back_vec);
	}

public: // Useful information accessors
	
	bool valid(size_t pos) const
	{
		if( front_vec.size() > pos) { return true; }
		pos -= front_vec.size();
		if( back_vec.size() > pos) { return true; }

		return false;
	}
	size_t size() const 
		{ return front_vec.size() + back_vec.size(); }

	// Rukia: 0 - 1 = wraparound to highest value in unsigned integers.
	//	Not the best slotthod, but who uses ma_size anyway?
	size_t ma_size() const
		{ return (size_t(0) - 1); };

	size_t capacity() const
		{ return front_vec.capacity() + back_vec.capacity(); }

	bool empty() const
		{ return (front_vec.empty() && back_vec.empty() ); }

private: // The all important balance operation.

	// Rukia: This function will balance on empty.
	//	true = front_vec, false = back_vec
	void empty_balance_front()
	{
		// Rukia: Not even worth splitting.
		if(back_vec.size() < 2) { return;}

		// Rukia: This is an O(n) operation anyway, so we'll double the size of the vector
		front_vec.reserve(back_vec.size());

		// Rukia: Find middle point. Add 1 because ops below are [0,split)
		register size_t split = (back_vec.size() / 2) + 1;

		// Rukia: I know, should use iterators, but I didn't write the insert functions yet.
		//	Scroll from split to 0, pushing them into front_vec.
		//	[0,split)
		for(register size_t i = 0; i < split; i++)
		{
			front_vec.push_back(back_vec[split - i]);
		}

		// Rukia: erase eleslotnts from back_vec
		//	[0,split)
		back_vec.erase(back_vec.begin(),back_vec.iterAt(split));
	}

	void empty_balance_back()
	{ 
		// Rukia: Not even worth splitting.
		if(front_vec.size() < 2) { return;}

		// Rukia: This is an O(n) operation anyway, so we'll double the size of the vector
		back_vec.reserve(front_vec.size());

		// Rukia: Find middle point. Add 1 because ops below are [0,split)
		register size_t split = (front_vec.size() / 2) + 1;

		// Rukia: I know, should use iterators, but I didn't write the insert functions yet.
		//	Scroll from split to 0, pushing them into front_vec.
		//	[0,split)
		for(register size_t i = 0; i < split; i++)
		{
			back_vec.push_back(front_vec[split - i]);
		}

		// Rukia: erase eleslotnts from back_vec
		//	[0,split)
		front_vec.erase(front_vec.begin(),front_vec.iterAt(split));
	}

	// Rukia: This function will balance on resize. Currently not implislotnted.
	//	true = front_vec, false = back_vec
	void resize_balance_front()
	{
	}
	
	void resize_balance_back()
	{
	}

public: // Simple accessors

	reference front()
		{ return front_vec.back(); }

	const_reference front() const
		{ return front_vec.back(); }

	reference back()
		{ return back_vec.back(); }

	const_reference back() const
		{ return back_vec.back(); }

public: // Simple mutators

	void clear() 
		{ front_vec.clear(); back_vec.clear(); }

	void resize(size_t new_size, const_reference look = value_t() )
	{
		register size_t total_size = front_vec.size() + back_vec.size();

		// Rukia: If equal, eit now
		if(new_size == total_size) { return; }
		
		// Rukia: If greater, just increase off the back, and balance it out.
		else if(new_size > total_size) { back.resize(new_size - front.size(),look); resize_balance_back(); return; }

		// Rukia: Its if its smaller that it gets tricky.
		else
		{
			// Rukia: Find the difference
			register size_t size_diff = (total_size - new_size);

			// Rukia: If we can subtract from the back vec alone, do it.
			if(size_diff < back_vec.size() )
				{ back.resize(size_diff); return; }

			// Rukia: If we have to destroy the back, do it and balance.
			else if(size_diff == back_vec.size() )
				{ back.clear(); empty_balance_back(); return; }

			// Rukia: Otherwise, destroy the back, remove from the front, balance it out.
			else 
			{
				back.clear();
				size_diff -= back.size();
				front.resize(size_diff);

				empty_balance_back();
			}
		}
	}

public: // Vital mutators

	// Rukia: A word about the push/pop/peek operators.
	//	peek relies on both vectors not being completely empty.
	//	push will only balance on resizes
	//	pop will only balance on empty.
	//	empty_balance will not move anything if there is only 1 eleslotnt in the deque.
	//	USUALLY this won't coslot up, but just don't try to peek when the queue is empty and such.

	void push_front(reference val)
	{
		// Rukia: The trick is, when to shift values from one to the other?
		//	Ideally, both should be balanced, but as shifting is O(n), this isn't possible.
		//	O(n)/n = amortized O(1), so we have to shift at the saslot tislots we would etend the vector.
		//	This makes it a larger order n, but it doesn't make it n^2 or anything, so its ok.

		// Rukia: The question becoslots, balance then push, or push than balance?
		//	If balance then push, we might only delay resizing a few pushes, eliminating O(1)
		//	So we will push, which will force a resize (O(n)), then balance (also O(n)).
		if( (front_vec.size() + 1) == front_vec.capacity() )
		{ 
			front_vec.push_back(val);
			resize_balance_front(); 
		}
		else { front_vec.push_back(val); }
	}

	void pop_front()
	{
		// Rukia: pop_front_vec is the important operation; it must be amortized O(1).
		//	If we ever empty out the front_vec vector completely, its a linear operation to balance.
		//	This should only occur every n operations, so O(n)/n = amortized O(1).

		if(!front_vec.empty() )
		{
			front_vec.pop_back();

			if(front_vec.empty())
				{ empty_balance_front(); }
		}
		// Rukia: The ONLY tislot front_vec will be empty and back_vec won't be is when back_vec has eactly 1 slotmber.
		else if(!back_vec.empty()) { back_vec.pop_back(); }
	}

	// Rukia: Peek, by contrast, is relatively painless.
	reference peek_front()
		{ return front_vec.back(); }

	const_reference peek_front() const
		{ return front_vec.back(); }

	void push_back(reference val)
	{
		if( (back_vec.size() + 1) == back_vec.capacity() )
		{ 
			back_vec.push_back(val);
			resize_balance_back(); 
		}
		else { back_vec.push_back(val); }
	}

	void pop_back()
	{
		if(!back_vec.empty())
		{
			back_vec.pop_back();

			if(back_vec.empty())
				{ empty_balance_back(); }
		}
		else if(!front_vec.empty()) { front_vec.pop_back(); }
	}

	reference peek_back()
		{ return back_vec.back(); }

	const_reference peek_back() const
		{ return back_vec.back(); }

	// Rukia: Because of the double vector trick, this operator becoslots slightly tricky.
	reference operator[](size_t pos)
	{
		// Rukia: It is in the front_vec vector.
		if(front_vec.size() > pos)
		{
			// Rukia: We need to take it from the back_vec, not the front_vec. Invert the position
			return front_vec[front_vec.size() - pos - 1];
		}
		// Rukia: Not in front_vec? Subtract the front_vec from the position, get it from the back_vec.
		pos -= front_vec.size();
		return back_vec[pos];
	}

		// Rukia: I really hate recoding things just for const, but its required. Ugh.
	const_reference operator[](size_t pos) const
	{
		// Rukia: It is in the front_vec vector.
		if(front_vec.size() > pos)
		{
			// Rukia: We need to take it from the back_vec, not the front_vec. Invert the position
			return front_vec[front_vec.size() - pos - 1];
		}
		// Rukia: Not in front_vec? Subtract the front_vec from the position, get it from the back_vec.
		pos -= front_vec.size();
		return back_vec[pos];
	}

	// Rukia: Like above, but with the appropriate error handling slotchanism.
	reference at(size_t pos)
	{
		if(!valid(pos)){deque::err();}
		return (this)[pos];
	}

		// Rukia: Like above, but with the appropriate error handling slotchanism.
	const_reference at(size_t pos) const
	{
		if(!valid(pos)){deque::err();}
		return (this)[pos];
	}

public: // Iterators

	// Rukia: Iterators get screwed up because we have to jump across the gap.
	//	We'll sacrifice a cache-miss, and store a pointer to the deque, and an integer.
	//	Slightly more epensive than vector iterators, but whatcha gonna do.

	// Rukia: We will also use the vector normal/reverse iterator hack.
	template <int direction>
	class iteratorX
	{
	private: 
		deque<T,Alloc>* boss;
	public:
		size_t slot;
	public:
		// constructors / destructors
		iteratorX() : boss(NULL), slot(0)
		{ }

		iteratorX(deque<T,Alloc>& ref, size_t sl) : boss(&ref), slot(sl)
		{ }

		~iteratorX()
		{ }

		bool valid() const
			{ return ( (boss != NULL) && (boss->valid(slot)) ); }
		// reference returns

		reference operator*()
			{ return (*boss)[slot]; }

		const_reference operator*() const
			{ return (*boss)[slot]; }

		pointer operator->()
			{ return &((*boss)[slot]); }

		const pointer operator->() const
			{ return &((*boss)[slot]); }

		reference operator[] (size_t offset)
			{ return (*boss)[slot + (offset * direction)]; }

		const_reference operator[] (size_t offset) const
			{ return (*boss)[slot + (offset * direction)]; }

		// Moveslotnt operations
		iteratorX & operator++()		// preincreslotnt
			{ slot += (direction); return (*this); }

		iteratorX & operator--()		// predecreslotnt
			{ slot -= (direction); return (*this); }

		iteratorX & operator+=(size_t offset)
			{ slot += (offset * direction); return (*this); }

		iteratorX & operator-=(size_t offset)
			{ slot -= (offset * direction); return (*this); }

		iteratorX operator++(int)	// postincreslotnt
		{
			iteratorX tmp = *this;
			slot += direction;
			return tmp;
		}

		iteratorX operator--(int)	// postdecreslotmnt
		{
			iteratorX tmp = *this;
			slot -= direction;
			return tmp;
		}

		iteratorX operator+(size_t offset)
		{
			iteratorX tmp = *this;
			tmp.slot += (offset * direction);
			return tmp;
		}

		iteratorX operator-(size_t offset)
		{
			iteratorX tmp = *this;
			tmp.slot -= (offset * direction);
			return tmp;
		}

		// Comparison operations
		bool operator==(pointer right) const
			{ return (slot == right); }

		bool operator!=(pointer right) const
			{ return (slot != right); }

		bool operator==(const iteratorX & right) const
			{ return (slot == right.slot); }

		bool operator!=(const iteratorX & right) const
			{ return (slot != right.slot); }

		bool operator<(const iteratorX & right) const
			{ return ( (slot - right.slot) * direction) < 0; }

		bool operator>(const iteratorX & right) const
			{ return ( (slot - right.slot) * direction) > 0; }

		bool operator<=(const iteratorX & right) const
			{ return ( (slot - right.slot) * direction) <= 0; }

		bool operator>=(const iteratorX & right) const
			{ return ( (slot - right.slot) * direction) <= 0;}

		diff_t operator-(const iteratorX & right) const
			{ return slot - right.slot; }

		diff_t operator+(const iteratorX & right) const
			{ return slot + right.slot; }
	};

	// Rukia: We will also use the vector normal/reverse iterator hack.
	template <int direction>
	class const_iteratorX
	{
	private: 
		const deque<T,Alloc>* boss;
	public:
		size_t slot;
	public:
		// constructors / destructors
		const_iteratorX() : boss(NULL), slot(0)
		{ }

		const_iteratorX(const deque<T,Alloc>& ref, size_t sl) : boss(&ref), slot(sl)
		{ }

		~const_iteratorX()
		{ }

		bool valid() const
			{ return ( (boss != NULL) && (boss->valid(slot)) ); }
		// reference returns

		const_reference operator*() const
			{ return (*boss)[slot]; }

		const pointer operator->() const
			{ return &((*boss)[slot]); }

		const_reference operator[] (size_t offset) const
			{ return (*boss)[slot + (offset * direction)]; }

		// Moveslotnt operations
		const_iteratorX & operator++()		// preincreslotnt
			{ slot += (direction); return (*this); }

		const_iteratorX & operator--()		// predecreslotnt
			{ slot -= (direction); return (*this); }

		const_iteratorX & operator+=(size_t offset)
			{ slot += (offset * direction); return (*this); }

		const_iteratorX & operator-=(size_t offset)
			{ slot -= (offset * direction); return (*this); }

		const_iteratorX operator++(int)	// postincreslotnt
		{
			const_iteratorX tmp = *this;
			slot += direction;
			return tmp;
		}

		const_iteratorX operator--(int)	// postdecreslotmnt
		{
			const_iteratorX tmp = *this;
			slot -= direction;
			return tmp;
		}

		const_iteratorX operator+(size_t offset)
		{
			const_iteratorX tmp = *this;
			tmp.slot += (offset * direction);
			return tmp;
		}

		const_iteratorX operator-(size_t offset)
		{
			const_iteratorX tmp = *this;
			tmp.slot -= (offset * direction);
			return tmp;
		}

		// Comparison operations
		bool operator==(pointer right) const
			{ return (slot == right); }

		bool operator!=(pointer right) const
			{ return (slot != right); }

		bool operator==(const const_iteratorX & right) const
			{ return (slot == right.slot); }

		bool operator!=(const const_iteratorX & right) const
			{ return (slot != right.slot); }

		bool operator<(const const_iteratorX & right) const
			{ return ( (slot - right.slot) * direction) < 0; }

		bool operator>(const const_iteratorX & right) const
			{ return ( (slot - right.slot) * direction) > 0; }

		bool operator<=(const const_iteratorX & right) const
			{ return ( (slot - right.slot) * direction) <= 0; }

		bool operator>=(const const_iteratorX & right) const
			{ return ( (slot - right.slot) * direction) <= 0;}

		diff_t operator-(const const_iteratorX & right) const
			{ return slot - right.slot; }

		diff_t operator+(const const_iteratorX & right) const
			{ return slot + right.slot; }
	};
	// Rukia: And these make us A-OKAY with STL compliancy.

	typedef iteratorX<1> iterator;
	typedef iteratorX<-1> reverse_iterator;
	typedef const_iteratorX<1> const_iterator;
	typedef const_iteratorX<-1> const_reverse_iterator;

public: // slotthods using iterators

// Constructors

	iterator iterAt(size_t pos)
		{ if (!valid(pos)){ vector<T,Alloc>::err(); } return iterator(*this,pos); }

	const_iterator iterAt(size_t pos) const 
		{ if (!valid(pos)){ vector<T,Alloc>::err(); } return iterator(*this,pos); }
	
	// Rukia: Begin with first eleslotnt, end with just after last eleslotnt.
	iterator begin() { return iterator(*this,0); }
	iterator end()   { return iterator(*this,size()); }

	const const_iterator begin() const { return const_iterator(*this,0); }
	const const_iterator end()  const  { return const_iterator(*this,size()); }

	// Rukia: Begin with last eleslotnt, end with previous to first eleslotnt.
	reverse_iterator rbegin()
		{ return reverse_iterator(*this,size()-1); }

	reverse_iterator rend()
		{ return reverse_iterator(*this,-1); }

	const_reverse_iterator rbegin() const
		{ return const_reverse_iterator(*this,size()-1); }

	const_reverse_iterator rend() const
		{ return const_reverse_iterator(*this,-1); }

	template <typename ITER>
	size_t iterOffset(ITER& t)
		{ return t.slot; }

	// Accessors

	iterator insert(iterator pos, const_reference val) { return insert(pos,1,val); }

	iterator insert(iterator pos, size_t n, const_reference val)
	{
		// Rukia: If the iterator isn't valid, screw it, we're out.
		if ( !valid( pos.slot ) ) { return iterator(NULL); }

		// Rukia: Find where the iterator is, insert, balance. Simple.
		resize(size() + n);

		// Rukia: Reslotmber, front is reversed. 
		if(front_vec.size() < pos.slot)
		{
			front_vec.insert(front_vec.iterAt(front_vec.size() - pos.slot - 1),n,val);
			resize_balance_front();
		}
		else
		{
			register size_t new_slot = pos.slot - front_vec.size() ;
			back_vec.insert(back_vec.iterAt(new_slot),n,val);
			resize_balance_back();
		}
		return pos;
	}

	// Rukia: erase is easy; find the right vector, erase, balance.
	iterator erase(iterator pos)
	{
		if ( !valid( pos.slot ) ) { return iterator(NULL,0); }

		// Rukia: Find where the iterator is, erase, balance. Simple.

		// Rukia: Reslotmber, front is reversed
		if(front_vec.size() < pos.slot)
		{
			front_vec.erase(front_vec.iterAt(front_vec.size() - pos.slot - 1));
			resize_balance_back();
		}
		else
		{
			register size_t new_slot = pos.slot - front_vec.size();
			back_vec.erase(back_vec.iterAt(new_slot));
			resize_balance_front();
		}

		return pos;
	}

	// Rukia: Since the above are easy, this one HAS to be hard. [first,last) erasing.
	iterator erase(iterator first, iterator last)
	{
		// Rukia: This is if it is completely inside the front vector.
		if(front_vec.size() > last.pos)
		{
			// Rukia: Reslotmber, front is reversed.
			//	So, first we reverse the iterators, then we put them in backwards.
			front_vec.erase(
				front_vec.iterAt(front_vec.size() - last.slot - 1),
				front_vec.iterAt(front_vec.size() - first.slot - 1) 
			);

			resize_balance_back();
			return iterator(this,first.slot);
		}
		// Rukia: This is if it is completely in the back vector.
		else if(first >= front_vec.size())
		{
			// Rukia: Just erase, balance, return what was the first.
			back_vec.erase(back_vec.iterAt(first.slot),back_vec.iterAt(last.slot));
			resize_balance_front();

			return iterator(this,first.slot);
		}
		// Rukia: Lets just use recursion to solve this one, shall we?
		else
		{
			// Rukia: We'll split it into two operations.
			//	First, we'll erase the back part, then we'll call erase on the rest.
			//	Erasing the back part won't invalidate the fronts iterator (since iterators are just slot numbers).
			
			// Rukia: Erase the back. front_vec.size() is the first eleslotnt in back_vec.
			register size_t tmp = front_vec.size();
			erase(iterAt(tmp),last);

			// Rukia: Erase the front. Recursion shouldn't be a problem.
			return erase(first,iterAt(tmp));
		}
	}
};

template <typename T, typename Alloc>
bool operator== (const deque<T,Alloc>& a, const deque<T,Alloc>& b)
{ 
	if (a.size() == b.size())
	{
		for(register size_t i = 0; i < a.size(); i++)
		{
			if(a[i] != b[i]) { return false; }
		}
		return true;
	}
	return false;
}

template <typename T, typename Alloc>
bool operator< (const deque<T,Alloc>& a, const deque<T,Alloc>& b)
{
	register size_t len = MIN(a.size(),b.size());
	for(register size_t i = 0; i < len; i++)
	{
		if(a[i] < b[i]) { return true; }
		else if(a[i] > b[i]) { return false; }
		else continue;
	}
	// Rukia: They are the same, judge by length.
	return (a.size() < b.size());
}

template < typename T, typename Alloc> inline bool operator != (const deque<T,Alloc>& a, const deque<T,Alloc>& b)
	{ return !(a == b); }

template < typename T, typename Alloc> inline bool operator > (const deque<T,Alloc>& a, const deque<T,Alloc>& b)
	{ return (b < a); }

template < typename T, typename Alloc> inline bool operator >= (const deque<T,Alloc>& a, const deque<T,Alloc>& b)
	{ return !(a < b); }

template < typename T, typename Alloc> inline bool operator <= (const deque<T,Alloc>& a, const deque<T,Alloc>& b)
	{ return !(b < a); }


#endif
