#ifndef __VECTOR___
#define __VECTOR___

// Rukia: My allocators
#include "alloc.h"

// Rukia: Container helper macros. Make programming a bit cleaner, but looking at the code a bit harder.
#include "container_helpers.h"

// Rukia: C libraries. Cross platform compatible.
#include <math.h>
#include <stdlib.h>
#include <assert.h>

#define MIN(a,b)    (((a) < (b)) ? (a) : (b))

// Rukia: My personal vector class. Nothing special, STL compliant, but also cross platform compatible. Enjoy.
//	As it is STL compliant, merely look up the STL vector for functions.
//	However, it does not use eceptions, nor does it do a lot of error handling. 
//	This is because it should be up to the programmer to do error handling for a lot of instances where I could do it.
//	So, don't try to get back() when the vector is empty, etc etc.
//	Adding checks would be trivial, but the cost would be non-trivial.

// Added functions:
// bool valid(size_t pos) | returns true if the position is inside of the vectors valid buffer
// void err() | this vector doesn't throw ecepts, so its platform compat. Modify err() to return errors for you.

// Rukia: Why vector? Because vector is taken, and Vector refers to math vectors.
template <typename T, typename Alloc = MAlloc<> >
class vector
{
public: 
	CONTAINER_TYPEDEF;
	static void err() { assert(0); }
private:
	pointer buffer;
	size_t used_size;
	size_t total_size;

public: // Constructors / Deconstructors
	// Rukia: Default constructor
	vector() : buffer(NULL), used_size(0), total_size(0)
	{ }

	 // Rukia: We have TWO to prevent stupid template overloading.
	vector(size_t elements, const_reference look = value_t()) : buffer(NULL), used_size(elements), total_size(0)
		{ scale(elements,GROW,look);}

	vector(const pointer elements, size_t amount) : buffer(NULL), used_size(0), total_size(0)
	{
		scale(amount,GROW);
		for(register size_t i = 0; i < amount; i++)
		{
			(*this)[i] = elements[i];
		}
		used_size = amount;
	}

	// Rukia: This can cause crazy stupid errors. Just use casts. I won't tell ^~
	template <class InputIterator>
	vector(InputIterator first, InputIterator last) : buffer(NULL), used_size(0), total_size(0)
	{
		for(; first != last; first++)
		{
			insert(end(),*first);
		}
	}

// Deconstructor
	~vector()
	{ clear(); }
// Copy Constructors
	vector(const vector& copy) : 
		buffer(NULL),
		used_size(copy.used_size), 
		total_size(0)
	{
		scale(copy.used_size,GROW);
		for(register size_t i = 0; i < copy.used_size; i++)
		{
			(*this)[i] = copy[i];
		}
	}

// Assignment
	vector& operator=(const vector& copy)
	{
		scale(copy.used_size,SCALE);
		for(register size_t i = 0; i < copy.used_size; i++)
		{
			(*this)[i] = copy[i];
		}
		used_size = copy.used_size;

		return *this;
	}


	void swap(vector& copy)
	{
		scale(copy.used_size,SCALE);
		for(register size_t i = 0; i < copy.used_size; i++)
		{
			(*this)[i] = copy[i];
		}
		used_size = copy.used_size;
	}

public: // Basic Accessor methods
	size_t size() const 
		{ return used_size; }

	// Rukia: 0 - 1 = wraparound to highest value in unsigned integers.
	//	Not the best method, but who uses ma_size anyway?
	size_t ma_size() const
		{ return (size_t(0) - 1); };

	size_t capacity() const
		{ return total_size; }

	bool valid(size_t pos) const
		{ return (pos < used_size); }

	bool valid(pointer pos) const
		{ return ( (pos >= buffer) && (pos < (buffer + used_size) ) ); }

private: // Hidden mutator methods

	void grow()
	{
		buffer = More<T,Alloc>(buffer,total_size,total_size * 2,value_t());
		total_size *= 2;
	}

	enum { SHRINK = -1, SCALE = 0, GROW = 1 };

	// Rukia: true = grow, false = shrink
	void scale(size_t point, int way = GROW, value_t look = value_t())
	{
		switch(way)
		{
			case SHRINK: if( total_size <= point) { return; }; break;
			case SCALE: if(total_size == point) { return; }; break;
			case GROW: if(total_size >= point) { return; }; break;
		}

		// Rukia: We will keep the size of the vector in multiple of 2's via this method.
		//	Take log2(point), round up, find 2^that, grow to there.
		//	Why? To maintain certain requirements for O(1) amortized on certain operations.
		double perc = log((double)point) / log((double)2);
		perc = ceil(perc);
		perc = pow((double)2,perc);

		// Rukia: Need to make it NULL safe. More -> realloc -> deletes on resize to 0.
		if(buffer == NULL) { buffer = New<T,Alloc>(static_cast<size_t>(perc),look); }
		else buffer = More<T,Alloc>(buffer,total_size,static_cast<size_t>(perc),look);
		total_size = static_cast<size_t>(perc);
	}

	void shrink()
	{
		// Rukia: If we are less than a third full, shrink the buffer.
		//	Why not less than half? (push;pop) results in a growth, than a shrink. Bad performance.
		if((used_size * 3) < total_size)
		{
			// Rukia: So what should we shrink it to?
			//	used_size * 2 should work nicely. stomp will force it all the way down if needed.
			scale(used_size * 2, GROW);
		}
	}

	// Rukia: Stomp jumps on the vector till all memory is used.
	void stomp()
	{
		buffer = More<T,Alloc>(buffer,total_size,used_size,value_t());
		total_size = used_size;
	}

	public: // vector iterator class

	// Rukia: I don't feel like implimenting reverse iterators when they are almost IDENTICAL.
	//	So, we'll do this, and hope it works.

	template <int direction>
	class iteratorX
	{

	private: 
		pointer me;
	public:
		// constructors / destructors
		iteratorX() : me(NULL)
		{ }

		iteratorX(pointer ptr) : me(ptr)
		{ }

		~iteratorX()
		{ }

		bool valid() const
			{ return (me != NULL); }

		// Pointer returns
		pointer base() 
			{ return me; }

		const pointer base() const
			{ return me; }
		
		// reference returns

		reference operator*()
			{ return *me; }

		pointer operator->()
			{ return me; }

		reference operator[] (size_t offset)
			{ return (*(*this + (offset * direction) )); }
		
		const_reference operator*() const
			{ return *me; }

		const pointer operator->() const
			{ return me; }

		const_reference operator[] (size_t offset) const
			{ return (*(*this + (offset * direction) )); }

		// Movement operations
		iteratorX & operator++()		// preincrement
			{ me += direction; return (*this); }

		iteratorX & operator--()		// predecrement
			{ me -= direction; return (*this); }

		iteratorX & operator+=(size_t offset)
			{ me += (offset * direction); return (*this); }

		iteratorX & operator-=(size_t offset)
			{ me -= (offset * direction); return (*this); }

		iteratorX operator++(int)	// postincrement
		{
			iteratorX tmp = *this;
			me += direction;
			return tmp;
		}

		iteratorX operator--(int)	// postdecrememnt
		{
			iteratorX tmp = *this;
			me -= direction;
			return tmp;
		}

		iteratorX operator+(size_t offset) const
		{
			iteratorX tmp(*this);
			tmp.me += (offset * direction);
			return tmp;
		}

		iteratorX operator-(size_t offset) const
		{
			iteratorX tmp(*this);
			tmp.me -= (offset * direction);
			return tmp;
		}

		// Comparison operations
		bool operator==(pointer right) const
			{ return (me == right); }

		bool operator!=(pointer right) const
			{ return (me != right); }

		bool operator==(const iteratorX & right) const
			{ return (me == right.me); }

		bool operator!=(const iteratorX & right) const
			{ return (me != right.me); }

		bool operator<(const iteratorX & right) const
			{ return ( (me - right.me) * direction) < 0; }

		bool operator>(const iteratorX & right) const
			{ return ( (me - right.me) * direction) > 0; }

		bool operator<=(const iteratorX & right) const
			{ return ( (me - right.me) * direction) <= 0; }

		bool operator>=(const iteratorX & right) const
			{ return ( (me - right.me) * direction) <= 0;}

		diff_t operator-(const iteratorX & right) const
			{ return me - right.me; }

		diff_t operator+(const iteratorX & right) const
			{ return me + right.me; }
	};

	template <int direction>
	class const_iteratorX
	{

	private: 
		pointer const me;
	public:
		// constructors / destructors
		const_iteratorX() : me(NULL)
		{ }

		const_iteratorX(pointer const ptr) : me(ptr)
		{ }

		~const_iteratorX()
		{ }

		bool valid() const
			{ return (me != NULL); }

		const pointer base() const
			{ return me; }
		
		// reference returns
		
		const_reference operator*() const
			{ return *me; }

		const pointer operator->() const
			{ return me; }

		const_reference operator[] (size_t offset) const
			{ return (*(*this + (offset * direction) )); }

		// Movement operations
		const_iteratorX & operator++()		// preincrement
			{ me += direction; return (*this); }

		const_iteratorX & operator--()		// predecrement
			{ me -= direction; return (*this); }

		const_iteratorX & operator+=(size_t offset)
			{ me += (offset * direction); return (*this); }

		const_iteratorX & operator-=(size_t offset)
			{ me -= (offset * direction); return (*this); }

		const_iteratorX operator++(int)	// postincrement
		{
			const_iteratorX tmp = *this;
			me += direction;
			return tmp;
		}

		const_iteratorX operator--(int)	// postdecrememnt
		{
			const_iteratorX tmp = *this;
			me -= direction;
			return tmp;
		}

		const_iteratorX operator+(size_t offset) const
		{
			const_iteratorX tmp(*this);
			tmp.me += (offset * direction);
			return tmp;
		}

		const_iteratorX operator-(size_t offset) const
		{
			const_iteratorX tmp(*this);
			tmp.me -= (offset * direction);
			return tmp;
		}

		// Comparison operations
		bool operator==(pointer right) const
			{ return (me == right); }

		bool operator!=(pointer right) const
			{ return (me != right); }

		bool operator==(const const_iteratorX & right) const
			{ return (me == right.me); }

		bool operator!=(const const_iteratorX & right) const
			{ return (me != right.me); }

		bool operator<(const const_iteratorX & right) const
			{ return ( (me - right.me) * direction) < 0; }

		bool operator>(const const_iteratorX & right) const
			{ return ( (me - right.me) * direction) > 0; }

		bool operator<=(const const_iteratorX & right) const
			{ return ( (me - right.me) * direction) <= 0; }

		bool operator>=(const const_iteratorX & right) const
			{ return ( (me - right.me) * direction) <= 0;}

		diff_t operator-(const const_iteratorX & right) const
			{ return me - right.me; }

		diff_t operator+(const const_iteratorX & right) const
			{ return me + right.me; }
	};


	// Rukia: And these make us A-OKAY with STL compliancy.

	typedef iteratorX<1> iterator;
	typedef iteratorX<-1> reverse_iterator;
	typedef const_iteratorX<1> const_iterator;
	typedef const_iteratorX<-1> const_reverse_iterator;

public: // Methods using iterators

// Constructors

	iterator iterAt(size_t pos)
		{ if (!valid(pos)){ vector::err(); } return iterator(buffer + pos); }

	const_iterator iterAt(size_t pos) const 
		{ if (!valid(pos)){ vector::err(); } return iterator(buffer + pos); }
	
	// Rukia: Begin with first element, end with just after last element.
	iterator begin() { return iterator(buffer); }
	iterator end()   { return iterator(buffer+used_size); }

	const iterator begin() const { return iterator(buffer); }
	const iterator end()  const  { return iterator(buffer+used_size); }

	// Rukia: Begin with last element, end with previous to first element.
	reverse_iterator rbegin()
		{ return reverse_iterator(buffer + (used_size - 1)); }

	reverse_iterator rend()
		{ return reverse_iterator(buffer - 1); }

	const_reverse_iterator rbegin() const
		{ return reverse_iterator(buffer + (used_size - 1)); }

	const_reverse_iterator rend() const
		{ return reverse_iterator(buffer - 1); }

	template <typename ITER>
	size_t iterOffset(ITER& t)
		{ return t.base() - buffer; }

// Accessors

	// Rukia: HACK: Ugh. Slow. For some, it can be quadratic or worst.
	//	I'll fi this later. InputIterators should have a garantee on distance anyway. SGI agrees.
	template <typename InputIterator>
	iterator insert(iterator pos, InputIterator first, InputIterator last)
	{
		for(; first != last; first++)
		{
			pos = insert(pos,*first);
		}
	}

	iterator insert(iterator pos, const_reference val) 
	{ 
		size_t offset = iterOffset(pos);
		iterinsert(pos,1,val);
		return iterAt(offset);
	}

	void insert(iterator pos, size_t n, const_reference val)
	{
		// Rukia: If the iterator isn't valid, screw it, we're out.
		if ( !valid( pos.base() ) ) { return; }

		size_t offset = iterOffset(pos);
		resize(used_size + n);
		pos = iterator(buffer + offset);

		register reverse_iterator rpos(buffer + offset);
		for(register reverse_iterator i = rbegin(); i <= rpos; i++)
		{
			register reverse_iterator k = i + n;
			(*i) = (*k);
		}

		for(register iterator i = pos; i < (pos + n); i++)
		{
			*i = val;
		}
	}

	iterator erase(iterator pos)
	{
		if ( !valid( pos.base() ) ) { return iterator(NULL); }

		for(register iterator i = pos; i != end(); i++)
		{
			register iterator k = i + 1;
			*i = *k;
		}

		size_t offset = iterOffset(pos);
		pop_back();

		return iterator(buffer + offset);
	}

	// [first,last)
	iterator erase(iterator first, iterator last)
	{
		register iterator i = first;
		for(register iterator k = last; k != end(); i++, k++)
		{
			*i = *k;
		}

		used_size -= (last - first);

		size_t offset = iterOffset(first);
		shrink();

		return iterator(buffer + offset);
	}


public: // Basic mutator methods

	// Rukia: Directly access elements
	reference at(size_t n)
		{ if(!valid(n)){vector::err();} return *(buffer + n); }
	reference operator[] (size_t n) 
		{ return *(buffer + n); }
	reference front() 
		{ return *(buffer); }
	reference back() 
		{ return *(buffer + (used_size - 1) ); }

	const_reference at(size_t n) const
		{ if(!valid(n)){vector::err();} return *(buffer + n); }
	const_reference operator[] (size_t n) const
		{ return *(buffer + n); }
	const_reference front() const
		{ return *(buffer); }
	const_reference back() const
		{ return *(buffer + (used_size - 1) ); }

	void assign(size_t amount, value_t look = value_t())
	{
		vector<value_t> tmp(amount,look);
		*this = tmp;
	}

	void assign(vector& tmp)
		{ *this = tmp; }

	// Rukia: realloc deletes when resize to 0, and new[0] does nothing.
	void clear()
	{
		used_size = 0;
		stomp();
		buffer = NULL;
	}

	bool empty() const
	{
		return (used_size == 0);
	}

	// Rukia: Indirectly add element
	void push_back(reference element)
		{ scale(used_size + 1,GROW); *(buffer + used_size) = element; used_size++; }

	// Rukia: Indirectly delete an element
	void pop_back()
		{ (buffer + (used_size - 1) )->~value_t(); used_size--; shrink(); }

	// Rukia: Indirectly access an element
	reference peek_back()
		{ return *(buffer + (used_size - 1) ); }

	const_reference peek_back() const
		{ return *(buffer + (used_size - 1) ); }

	void resize(size_t new_size, value_t fill = value_t())
		{ scale(new_size,SCALE,fill); used_size = new_size; }

	void reserve(size_t new_size)
		{ scale(new_size,GROW);}

};

// Rukia: STL requires equality and less than comparison. All other comparisons can be derived from these two.
//	For the sake of convience, I've done the derivations.

// Rukia: Vector comparisons are the only hard ones; everything else derives from vectors really.
template <typename T, typename Alloc>
bool operator== (const vector<T,Alloc>& a, const vector<T,Alloc>& b)
{ 
	if (a.size() == b.size())
	{
		for(register size_t i = 0; i < a.size(); i++)
		{
			if(a[i] != b[i]) { return false; }
		}
		return true;
	}
	return false;
}

template <typename T, typename Alloc>
bool operator< (const vector<T,Alloc>& a, const vector<T,Alloc>& b)
{
	register size_t len = MIN(a.size(),b.size());
	for(register size_t i = 0; i < len; i++)
	{
		if(a[i] < b[i]) { return true; }
		else if(a[i] > b[i]) { return false; }
		else continue;
	}
	// Rukia: They are the same, judge by length.
	return (a.size() < b.size());
}

template < typename T, typename Alloc> inline bool operator != (const vector<T,Alloc>& a, const vector<T,Alloc>& b)
	{ return !(a == b); }

template < typename T, typename Alloc> inline bool operator > (const vector<T,Alloc>& a, const vector<T,Alloc>& b)
	{ return (b < a); }

template < typename T, typename Alloc> inline bool operator >= (const vector<T,Alloc>& a, const vector<T,Alloc>& b)
	{ return !(a < b); }

template < typename T, typename Alloc> inline bool operator <= (const vector<T,Alloc>& a, const vector<T,Alloc>& b)
	{ return !(b < a); }

#endif
